library angular2.test.http.backends.xhr_backend_spec;

import "package:angular2/test_lib.dart"
    show
        AsyncTestCompleter,
        afterEach,
        beforeEach,
        ddescribe,
        describe,
        expect,
        iit,
        inject,
        it,
        xit,
        SpyObject;
import "package:angular2/src/facade/async.dart" show ObservableWrapper;
import "package:angular2/src/http/backends/browser_xhr.dart" show BrowserXhr;
import "package:angular2/src/http/backends/xhr_backend.dart"
    show XHRConnection, XHRBackend;
import "package:angular2/di.dart" show bind, Injector;
import "package:angular2/src/http/static_request.dart" show Request;
import "package:angular2/src/http/static_response.dart" show Response;
import "package:angular2/src/http/headers.dart" show Headers;
import "package:angular2/src/facade/collection.dart" show Map;
import "package:angular2/src/http/base_request_options.dart"
    show RequestOptions, BaseRequestOptions;
import "package:angular2/src/http/base_response_options.dart"
    show BaseResponseOptions, ResponseOptions;
import "package:angular2/src/http/enums.dart" show ResponseTypes;

var abortSpy;
var sendSpy;
var openSpy;
var setRequestHeaderSpy;
var addEventListenerSpy;
var existingXHRs = [];
Response unused;
class MockBrowserXHR extends BrowserXhr {
  dynamic abort;
  dynamic send;
  dynamic open;
  dynamic response;
  String responseText;
  dynamic setRequestHeader;
  Map<String, Function> callbacks;
  num status;
  MockBrowserXHR() : super() {
    /* super call moved to initializer */;
    var spy = new SpyObject();
    this.abort = abortSpy = spy.spy("abort");
    this.send = sendSpy = spy.spy("send");
    this.open = openSpy = spy.spy("open");
    this.setRequestHeader = setRequestHeaderSpy = spy.spy("setRequestHeader");
    this.callbacks = new Map();
  }
  setStatusCode(status) {
    this.status = status;
  }
  setResponse(value) {
    this.response = value;
  }
  setResponseText(value) {
    this.responseText = value;
  }
  addEventListener(String type, Function cb) {
    this.callbacks[type] = cb;
  }
  dispatchEvent(String type) {
    this.callbacks[type]({});
  }
  build() {
    var xhr = new MockBrowserXHR();
    existingXHRs.add(xhr);
    return xhr;
  }
}
main() {
  describe("XHRBackend", () {
    var backend;
    var sampleRequest;
    beforeEach(() {
      var injector = Injector.resolveAndCreate([
        bind(ResponseOptions).toClass(BaseResponseOptions),
        bind(BrowserXhr).toClass(MockBrowserXHR),
        XHRBackend
      ]);
      backend = injector.get(XHRBackend);
      var base = new BaseRequestOptions();
      sampleRequest = new Request(
          base.merge(new RequestOptions(url: "https://google.com")));
    });
    afterEach(() {
      existingXHRs = [];
    });
    it("should create a connection", () {
      expect(() => backend.createConnection(sampleRequest)).not.toThrow();
    });
    describe("XHRConnection", () {
      it("should use the injected BaseResponseOptions to create the response",
          inject([AsyncTestCompleter], (async) {
        var connection = new XHRConnection(sampleRequest, new MockBrowserXHR(),
            new ResponseOptions(type: ResponseTypes.Error));
        ObservableWrapper.subscribe(connection.response, (res) {
          expect(res.type).toBe(ResponseTypes.Error);
          async.done();
        });
        existingXHRs[0].dispatchEvent("load");
      }));
      it("should complete a request", inject([AsyncTestCompleter], (async) {
        var connection = new XHRConnection(sampleRequest, new MockBrowserXHR(),
            new ResponseOptions(type: ResponseTypes.Error));
        ObservableWrapper.subscribe(connection.response, (res) {
          expect(res.type).toBe(ResponseTypes.Error);
        }, null, () {
          async.done();
        });
        existingXHRs[0].dispatchEvent("load");
      }));
      it("should call abort when disposed", () {
        var connection = new XHRConnection(sampleRequest, new MockBrowserXHR());
        connection.dispose();
        expect(abortSpy).toHaveBeenCalled();
      });
      it("should automatically call open with method and url", () {
        new XHRConnection(sampleRequest, new MockBrowserXHR());
        expect(openSpy).toHaveBeenCalledWith("GET", sampleRequest.url);
      });
      it("should automatically call send on the backend with request body", () {
        var body = "Some body to love";
        var base = new BaseRequestOptions();
        new XHRConnection(
            new Request(base.merge(new RequestOptions(body: body))),
            new MockBrowserXHR());
        expect(sendSpy).toHaveBeenCalledWith(body);
      });
      it("should attach headers to the request", () {
        var headers =
            new Headers({"Content-Type": "text/xml", "Breaking-Bad": "<3"});
        var base = new BaseRequestOptions();
        new XHRConnection(
            new Request(base.merge(new RequestOptions(headers: headers))),
            new MockBrowserXHR());
        expect(setRequestHeaderSpy).toHaveBeenCalledWith(
            "Content-Type", ["text/xml"]);
        expect(setRequestHeaderSpy).toHaveBeenCalledWith(
            "Breaking-Bad", ["<3"]);
      });
      it("should return the correct status code", inject([AsyncTestCompleter],
          (async) {
        var statusCode = 418;
        var connection = new XHRConnection(sampleRequest, new MockBrowserXHR(),
            new ResponseOptions(status: statusCode));
        ObservableWrapper.subscribe(connection.response, (res) {
          expect(res.status).toBe(statusCode);
          async.done();
        });
        existingXHRs[0].setStatusCode(statusCode);
        existingXHRs[0].dispatchEvent("load");
      }));
      it("should normalize IE's 1223 status code into 204", inject(
          [AsyncTestCompleter], (async) {
        var statusCode = 1223;
        var normalizedCode = 204;
        var connection = new XHRConnection(sampleRequest, new MockBrowserXHR(),
            new ResponseOptions(status: statusCode));
        ObservableWrapper.subscribe(connection.response, (res) {
          expect(res.status).toBe(normalizedCode);
          async.done();
        });
        existingXHRs[0].setStatusCode(statusCode);
        existingXHRs[0].dispatchEvent("load");
      }));
      it("should normalize responseText and response", inject(
          [AsyncTestCompleter], (async) {
        var responseBody = "Doge";
        var connection1 = new XHRConnection(
            sampleRequest, new MockBrowserXHR(), new ResponseOptions());
        var connection2 = new XHRConnection(
            sampleRequest, new MockBrowserXHR(), new ResponseOptions());
        ObservableWrapper.subscribe(connection1.response, (res) {
          expect(res.text()).toBe(responseBody);
          ObservableWrapper.subscribe(connection2.response, (ress) {
            expect(ress.text()).toBe(responseBody);
            async.done();
          });
          existingXHRs[1].dispatchEvent("load");
        });
        existingXHRs[0].setResponseText(responseBody);
        existingXHRs[1].setResponse(responseBody);
        existingXHRs[0].dispatchEvent("load");
      }));
    });
  });
}
